#!/usr/local/bin/python

#
# PMPP, a Poor Man's Pre Processor.
# by Kevin Pulo (kev@zip.com.au), 1999.
# Covered by the terms of the GNU GPL, as described in the file COPYING.
#
# Version 0.01
# $Id: pmpp.py,v 1.7 1999/02/21 00:31:59 kev Exp kev $
#

import sys, string, re

True = 1
true = True
False = 0
false = False

_template_file = "_template_file"
_out_file = "_out_file"
_header = "_header"
_header_file = "_header_file"
_footer = "_footer"
_footer_file = "_footer_file"

_required_gvars = [ _template_file ]
_mutually_exclusive_gvars = [ (_header, _header_file), (_footer, _footer_file) ]

# I'm too lazy to figure this out.  What I'm after is to strip off any
# leading pathname and trailing .py, similar to ${{0##*/}%.py} in bash or
# `basename $0 .py`
#_program_name = re.match(r"^(.*/)*(\.py)?$", sys.argv[0]).group(1)
_program_name = sys.argv[0]
_program_version = "0.01"



def warning(s):
	sys.stderr.write(_program_name + ": Warning: " + s + "\n")


def error(s, errorlevel = 1):
	sys.stderr.write(_program_name + ": Error: " + s + "\n")
	sys.exit(errorlevel)


def getkeyvalue(file):
	# At all times, currline is the line to next be processed.
	global currline

	# Skip any blank lines or commented lines.
	while (string.strip(currline) == "" or string.strip(currline)[0] == "#" or string.strip(currline)[0] == ";") and currline != "":
		currline = file.readline()

	# Since currline is the return from readline() if it == "" then it's eof.
	if currline == "":
		raise EOFError

	# This needs to check that the line can actually be split up like this
	# before going and doing it.
	key, value = re.split(r"=", currline, 1)

	# Strip key and value, so that the config file may have stuff like:
	# 'key = value' instead of 'key=value'
	key = string.strip(key)
	value = string.strip(value)

	# If the value has " as it's first character, then it's a multiline
	# value.  Strip off the initial ", add the \n back to the end and then
	# keep adding lines to value until we hit a line with just '.' on it.
	# Then take off the trailing \n from the last line.
	if value[0] == '"':
		value = value[1:] + "\n"
		currline = file.readline()
		while string.strip(currline) != "." and currline != "":
			value = value + currline
			currline = file.readline()
		# Remove the trailing '\n'.
		value = value[:len(value)-1]

	# Get the next line.
	currline = file.readline()

	return key, value



#
# Start of main program.
#


# Construct the list of config files to process.
if len(sys.argv) == 1:
	warning("No config files specified, reading config from stdin.")
	configfiles = [ "-" ]
else:
	configfiles = sys.argv[1:]


# Process each config file.
for configfile in configfiles:

	# Open the config file and read the first line.
	if configfile == "-":
		config = sys.stdin
	else:
		try:
			config = open(configfile)
		except IOError:
			warning("Unable to open config file '" + configfile + "', skipping...")
			continue
	currline = config.readline()


	# Read in initial setup.
	gvars = {}
	try:
		key, value = getkeyvalue(config)
	except EOFError:
		warning("Unexpected EOF in file '" + configfile + "', skipping...")
		continue

	if key != "global" and value != "start":
		warning("Parse error in config file '" + configfile + "', expected 'global=start', got '" + key + "=" + value + "', skipping...")
		continue
	else:
		try:
			key, value = getkeyvalue(config)
		except EOFError:
			warning("Unexpected EOF in file '" + configfile + "', skipping...")
			continue

	while key != "global" and value != "end":
		gvars[key] = value
		try:
			key, value = getkeyvalue(config)
		except EOFError:
			# This means we need at least one article.
			warning("Unexpected EOF in file '" + configfile + "', skipping...")
			continue


	# Check that all the required gvars have been listed.
	allthere = true
	for required_gvar in _required_gvars:
		if not gvars.has_key(required_gvar):
			warning("The required global variable '" + required_gvar + "' not specified in config file '" + configfile + "', skipping...")
			allthere = false
	if not allthere:
		continue

	# Check that no two mutually exclusive gvars are set.
	allok = true
	for mutexset in _mutually_exclusive_gvars:
		gvarsset = []
		# In each set of mutually exclusive gvars, count how many are set.
		for x in mutexset:
			if gvars.has_key(x):
				gvarsset.append(x)
		# And if it's more than one, we have a problem.
		if len(gvarsset) > 1:
			warning("The following mutually exclusive global variables are set: '" + `gvarsset` + "', as specified in config file '" + configfile + "', skipping...")
			allok = false
	if not allok:
		continue


	# Read in the individual articles.
	articles = []
	while true:
		article = {}
		try:
			key, value = getkeyvalue(config)
		except EOFError:
			# If there are no more articles, that's fine!!  :)
			break

		if key != "article" and value != "start":
			warning("Parse error in config file '" + configfile + "', expected 'article=start', got '" + key + "=" + value + "', skipping...")
			continue
		else:
			try:
				key, value = getkeyvalue(config)
			except EOFError:
				warning("Unexpected EOF in file '" + configfile + "', skipping...")
				continue

		while key != "article" and value != "end":
			article[key] = value
			try:
				key, value = getkeyvalue(config)
			except EOFError:
				warning("Unexpected EOF in file '" + configfile + "', skipping...")
				continue

		articles.append(article)

	config.close()


	# Open the template file and read it all in.
	try:
		templatefile = open(gvars[_template_file])
	except IOError:
		warning("Unable to open template file '" + gvars[_template_file] + "' specified by config file '" + configfile + "', skipping...")
		continue
	templatelines = templatefile.readlines()
	templatefile.close()


	# Open the output file.
	if not gvars.has_key(_out_file) or gvars[_out_file] == "-":
		outfile = sys.stdin
	else:
		try:
			outfile = open(gvars[_out_file], "w")
		except IOError:
			warning("Unable to open output file '" + gvars[_out_file] + "' (for writing) specified by config file '" + configfile + "', skipping...")
			continue


	# Only one of _header and _header_file should be set, so doing things
	# this way is safe.

	# If set, output the header string.
	if gvars.has_key(_header):
		outfile.write(gvars[_header] + "\n")

	# If set, output the header file.
	if gvars.has_key(_header_file):
		try:
			headerfile = open(gvars[_header_file])
		except IOError:
			warning("Unable to open header file '" + gvars[_header_file] + "' specified by config file '" + configfile + "', skipping...")
			continue
		# Will make this substitute global variables, to allow replacement
		# operators for global directives in the header and footer files.
		outfile.writelines(headerfile.readlines())


	# Output each of the articles.
	for article in articles:
		for line in templatelines:
			# Substitute any global variables.
			for key in gvars.keys():
				line = re.sub(r"&" + key + "&", gvars[key], line)
			# Substitute any article variables.
			for key in article.keys():
				line = re.sub(r"&" + key + "&", article[key], line)
			outfile.write(line)


	# Only one of _header and _header_file should be set, so doing things
	# this way is safe.

	# If set, output the footer string.
	if gvars.has_key(_footer):
		outfile.write(gvars[_footer] + "\n")

	# If set, output the footer file.
	if gvars.has_key(_footer_file):
		try:
			footerfile = open(gvars[_footer_file])
		except IOError:
			warning("Unable to open footer file '" + gvars[_footer_file] + "' specified by config file '" + configfile + "', skipping...")
			continue
		# Will make this substitute global variables, to allow replacement
		# operators for global directives in the header and footer files.
		outfile.writelines(footerfile.readlines())


	outfile.close()


